@@ -85,6 +85,9 @@ ContainerSpec:
 {{- if .ContainerWorkDir }}
  Dir:		{{ .ContainerWorkDir }}
 {{- end -}}
+{{- if .HasContainerInit }}
+ Init:		{{ .ContainerInit }}
+{{- end -}}
 {{- if .ContainerUser }}
  User: {{ .ContainerUser }}
 {{- end }}
@@ -92,11 +95,23 @@ ContainerSpec:
 Mounts:
 {{- end }}
 {{- range $mount := .ContainerMounts }}
-  Target = {{ $mount.Target }}
-   Source = {{ $mount.Source }}
-   ReadOnly = {{ $mount.ReadOnly }}
-   Type = {{ $mount.Type }}
+ Target:	{{ $mount.Target }}
+  Source:	{{ $mount.Source }}
+  ReadOnly:	{{ $mount.ReadOnly }}
+  Type:		{{ $mount.Type }}
 {{- end -}}
+{{- if .Configs}}
+Configs:
+{{- range $config := .Configs }}
+ Target:	{{$config.File.Name}}
+  Source:	{{$config.ConfigName}}
+{{- end }}{{ end }}
+{{- if .Secrets }}
+Secrets:
+{{- range $secret := .Secrets }}
+ Target:	{{$secret.File.Name}}
+  Source:	{{$secret.SecretName}}
+{{- end }}{{ end }}
 {{- if .HasResources }}
 Resources:
 {{- if .HasResourceReservations }}
@@ -200,6 +215,14 @@ func (ctx *serviceInspectContext) Labels() map[string]string {
 	return ctx.Service.Spec.Labels
 }
 
+func (ctx *serviceInspectContext) Configs() []*swarm.ConfigReference {
+	return ctx.Service.Spec.TaskTemplate.ContainerSpec.Configs
+}
+
+func (ctx *serviceInspectContext) Secrets() []*swarm.SecretReference {
+	return ctx.Service.Spec.TaskTemplate.ContainerSpec.Secrets
+}
+
 func (ctx *serviceInspectContext) IsModeGlobal() bool {
 	return ctx.Service.Spec.Mode.Global != nil
 }
@@ -352,6 +375,14 @@ func (ctx *serviceInspectContext) ContainerUser() string {
 	return ctx.Service.Spec.TaskTemplate.ContainerSpec.User
 }
 
+func (ctx *serviceInspectContext) HasContainerInit() bool {
+	return ctx.Service.Spec.TaskTemplate.ContainerSpec.Init != nil
+}
+
+func (ctx *serviceInspectContext) ContainerInit() bool {
+	return *ctx.Service.Spec.TaskTemplate.ContainerSpec.Init
+}
+
 func (ctx *serviceInspectContext) ContainerMounts() []mounttypes.Mount {
 	return ctx.Service.Spec.TaskTemplate.ContainerSpec.Mounts
 }
