@@ -10,9 +10,10 @@ import (
 	"time"
 
 	"github.com/docker/cli/cli/compose/types"
-	"github.com/gotestyourself/gotestyourself/assert"
-	is "github.com/gotestyourself/gotestyourself/assert/cmp"
+	"github.com/google/go-cmp/cmp/cmpopts"
 	"github.com/sirupsen/logrus"
+	"gotest.tools/assert"
+	is "gotest.tools/assert/cmp"
 )
 
 func buildConfigDetails(source map[string]interface{}, env map[string]string) types.ConfigDetails {
@@ -181,6 +182,23 @@ func TestLoad(t *testing.T) {
 	assert.Check(t, is.DeepEqual(sampleConfig.Volumes, actual.Volumes))
 }
 
+func TestLoadExtras(t *testing.T) {
+	actual, err := loadYAML(`
+version: "3.7"
+services:
+  foo:
+    image: busybox
+    x-foo: bar`)
+	assert.NilError(t, err)
+	assert.Check(t, is.Len(actual.Services, 1))
+	service := actual.Services[0]
+	assert.Check(t, is.Equal("busybox", service.Image))
+	extras := map[string]interface{}{
+		"x-foo": "bar",
+	}
+	assert.Check(t, is.DeepEqual(extras, service.Extras))
+}
+
 func TestLoadV31(t *testing.T) {
 	actual, err := loadYAML(`
 version: "3.1"
@@ -824,6 +842,9 @@ func TestFullExample(t *testing.T) {
 	assert.Check(t, is.DeepEqual(expectedConfig.Services, config.Services))
 	assert.Check(t, is.DeepEqual(expectedConfig.Networks, config.Networks))
 	assert.Check(t, is.DeepEqual(expectedConfig.Volumes, config.Volumes))
+	assert.Check(t, is.DeepEqual(expectedConfig.Secrets, config.Secrets))
+	assert.Check(t, is.DeepEqual(expectedConfig.Configs, config.Configs))
+	assert.Check(t, is.DeepEqual(expectedConfig.Extras, config.Extras))
 }
 
 func TestLoadTmpfsVolume(t *testing.T) {
@@ -1357,3 +1378,88 @@ networks:
 	assert.ErrorContains(t, err, "network.external.name and network.name conflict; only use network.name")
 	assert.ErrorContains(t, err, "foo")
 }
+
+func TestLoadNetworkWithName(t *testing.T) {
+	config, err := loadYAML(`
+version: '3.5'
+services:
+  hello-world:
+    image: redis:alpine
+    networks:
+      - network1
+      - network3
+
+networks:
+  network1:
+    name: network2
+  network3:
+`)
+	assert.NilError(t, err)
+	expected := &types.Config{
+		Filename: "filename.yml",
+		Version:  "3.5",
+		Services: types.Services{
+			{
+				Name:  "hello-world",
+				Image: "redis:alpine",
+				Networks: map[string]*types.ServiceNetworkConfig{
+					"network1": nil,
+					"network3": nil,
+				},
+			},
+		},
+		Networks: map[string]types.NetworkConfig{
+			"network1": {Name: "network2"},
+			"network3": {},
+		},
+	}
+	assert.DeepEqual(t, config, expected, cmpopts.EquateEmpty())
+}
+
+func TestLoadInit(t *testing.T) {
+	booleanTrue := true
+	booleanFalse := false
+
+	var testcases = []struct {
+		doc  string
+		yaml string
+		init *bool
+	}{
+		{
+			doc: "no init defined",
+			yaml: `
+version: '3.7'
+services:
+  foo:
+    image: alpine`,
+		},
+		{
+			doc: "has true init",
+			yaml: `
+version: '3.7'
+services:
+  foo:
+    image: alpine
+    init: true`,
+			init: &booleanTrue,
+		},
+		{
+			doc: "has false init",
+			yaml: `
+version: '3.7'
+services:
+  foo:
+    image: alpine
+    init: false`,
+			init: &booleanFalse,
+		},
+	}
+	for _, testcase := range testcases {
+		t.Run(testcase.doc, func(t *testing.T) {
+			config, err := loadYAML(testcase.yaml)
+			assert.NilError(t, err)
+			assert.Check(t, is.Len(config.Services, 1))
+			assert.Check(t, is.DeepEqual(config.Services[0].Init, testcase.init))
+		})
+	}
+}
