@@ -6,15 +6,15 @@ import (
 
 	"github.com/docker/docker/api/types"
 	"github.com/docker/docker/internal/test/daemon"
-	"github.com/gotestyourself/gotestyourself/assert"
-	"github.com/gotestyourself/gotestyourself/skip"
+	"gotest.tools/assert"
+	"gotest.tools/skip"
 )
 
 // Regression test for #35553
 // Ensure that a daemon with a log plugin set as the default logger for containers
 // does not keep the daemon from starting.
 func TestDaemonStartWithLogOpt(t *testing.T) {
-	skip.IfCondition(t, testEnv.IsRemoteDaemon(), "cannot run daemon when remote daemon")
+	skip.If(t, testEnv.IsRemoteDaemon, "cannot run daemon when remote daemon")
 	t.Parallel()
 
 	d := daemon.New(t)
