@@ -1,13 +1,13 @@
-package system
+package system // import "github.com/docker/docker/integration/system"
 
 import (
+	"context"
 	"fmt"
 	"testing"
 
-	"github.com/docker/docker/integration/internal/request"
-	"github.com/gotestyourself/gotestyourself/assert"
-	is "github.com/gotestyourself/gotestyourself/assert/cmp"
-	"golang.org/x/net/context"
+	"github.com/docker/docker/internal/test/request"
+	"gotest.tools/assert"
+	is "gotest.tools/assert/cmp"
 )
 
 func TestInfoAPI(t *testing.T) {
